<?php
define('FINANCIAL_ACCURACY', 1.0e-6);
define('FINANCIAL_MAX_ITERATIONS', 100);
define('FINANCIAL_SECS_PER_DAY', 24 * 60 * 60);
define('FINANCIAL_HALF_SEC', 0.5 / FINANCIAL_SECS_PER_DAY);






	/**
	 * XIRR
	 * Returns the internal rate of return for a schedule of cash flows
	 * that is not necessarily periodic. To calculate the internal rate
	 * of return for a series of periodic cash flows, use the IRR function.
	 * 
	 * Adapted from routine in Numerical Recipes in C, and translated
	 * from the Bernt A Oedegaard algorithm in C
	 * 
	 */
	function XIRR($values, $dates, $guess = 0.1)
	{
		if ((!is_array($values)) && (!is_array($dates))) return null;
		if (count($values) != count($dates)) return null;
		
		// create an initial bracket, with a root somewhere between bot and top
		$x1 = 0.0;
		$x2 = $guess;
		$f1 = XNPV($x1, $values, $dates);
		$f2 = XNPV($x2, $values, $dates);
		for ($i = 0; $i < FINANCIAL_MAX_ITERATIONS; $i++)
		{
			if (($f1 * $f2) < 0.0) break;
			if (abs($f1) < abs($f2)) {
				$f1 = XNPV($x1 += 1.6 * ($x1 - $x2), $values, $dates);
			} else {
				$f2 = XNPV($x2 += 1.6 * ($x2 - $x1), $values, $dates);
			}
		}
		
		if (($f1 * $f2) > 0.0) return null;
		
		$f = XNPV($x1, $values, $dates);
		if ($f < 0.0) {
			$rtb = $x1;
			$dx = $x2 - $x1;
		} else {
			$rtb = $x2;
			$dx = $x1 - $x2;
		}
		
		for ($i = 0;  $i < FINANCIAL_MAX_ITERATIONS; $i++)
		{
			$dx *= 0.5;
			$x_mid = $rtb + $dx;
			$f_mid = XNPV($x_mid, $values, $dates);
			if ($f_mid <= 0.0) $rtb = $x_mid;
			if ((abs($f_mid) < FINANCIAL_ACCURACY) || (abs($dx) < FINANCIAL_ACCURACY)) return $x_mid;
		}
		return null;
	}


	function XNPV($rate, $values, $dates)
	{
		if ((!is_array($values)) || (!is_array($dates))) return null;
		if (count($values) != count($dates)) return null;
		
		$xnpv = 0.0;
		for ($i = 0; $i < count($values); $i++)
		{
			$xnpv += $values[$i] / pow(1 + $rate, DATEDIFF('day', $dates[0], $dates[$i]) / 365);
		}
		return (is_finite($xnpv) ? $xnpv: null);
	}


	function DATEDIFF($datepart, $startdate, $enddate)
	{
		switch (strtolower($datepart)) {
			case 'yy':
			case 'yyyy':
			case 'year':
				$di = getdate($startdate);
				$df = getdate($enddate);
				return $df['year'] - $di['year'];
				break;
			case 'q':
			case 'qq':
			case 'quarter':
				die("Unsupported operation");
				break;
			case 'n':
			case 'mi':
			case 'minute':
				return ceil(($enddate - $startdate) / 60); 
				break;
			case 'hh':
			case 'hour':
				return ceil(($enddate - $startdate) / 3600); 
				break;
			case 'd':
			case 'dd':
			case 'day':
				return ceil(($enddate - $startdate) / 86400); 
				break;
			case 'wk':
			case 'ww':
			case 'week':
				return ceil(($enddate - $startdate) / 604800); 
				break;
			case 'm':
			case 'mm':
			case 'month':
				$di = getdate($startdate);
				$df = getdate($enddate);
				return ($df['year'] - $di['year']) * 12 + ($df['mon'] - $di['mon']);
				break;
			default:
				die("Unsupported operation");
		}
	}


	// IRR

	function IRR($values, $guess = 0.1)
	{
		if (!is_array($values)) return null;
		
		// create an initial bracket, with a root somewhere between bot and top
		$x1 = 0.0;
		$x2 = $guess;
		$f1 = NPV($x1, $values);
		$f2 = NPV($x2, $values);
		for ($i = 0; $i < FINANCIAL_MAX_ITERATIONS; $i++)
		{
			if (($f1 * $f2) < 0.0) break;
			if (abs($f1) < abs($f2)) {
				$f1 = NPV($x1 += 1.6 * ($x1 - $x2), $values);
			} else {
				$f2 = NPV($x2 += 1.6 * ($x2 - $x1), $values);
			}
		}
		if (($f1 * $f2) > 0.0) return null;
		
		$f = NPV($x1, $values);
		if ($f < 0.0) {
			$rtb = $x1;
			$dx = $x2 - $x1;
		} else {
			$rtb = $x2;
			$dx = $x1 - $x2;

		}
		
		for ($i = 0;  $i < FINANCIAL_MAX_ITERATIONS; $i++)
		{
			
			$dx *= 0.5;
			$x_mid = $rtb + $dx;
			$f_mid = NPV($x_mid, $values);
			if ($f_mid <= 0.0) $rtb = $x_mid;
			if ((abs($f_mid) < FINANCIAL_ACCURACY) || (abs($dx) < FINANCIAL_ACCURACY)) return $x_mid;
		}
		return null;
	}


		/**
	 * NPV
	 * Calculates the net present value of an investment by using a
	 * discount rate and a series of future payments (negative values)
	 * and income (positive values).
	 * 
	 *        n   /   values(i)  \
	 * NPV = SUM | -------------- |
	 *       i=1 |            i   |
	 *            \  (1 + rate)  /
	 * 
	 */
	function NPV($rate, $values)
	{
		if (!is_array($values)) return null;
		
		$npv = 0.0;
		for ($i = 0; $i < count($values); $i++)
		{
			$npv += $values[$i] / pow(1 + $rate, $i + 1);
		}
		return (is_finite($npv) ? $npv: null);
	}